@@ -1,4 +1,4 @@
-version = '0.19.1'
+version = '0.20'
 
 default_banner = "Rackspace Cloud Files %v using pyftpdlib %f ready."
 default_config_file = '/etc/ftpcloudfs.conf'
@@ -6,3 +6,8 @@ default_address = '127.0.0.1'
 default_port = 2021
 default_workers = 1
 cloudfiles_api_timeout = 30
+
+# keystone defaults
+default_ks_tenant_separator = '.'
+default_ks_service_type = 'object-store'
+default_ks_endpoint_type = 'publicURL'
